--*
--
-- OBJECTIVE:
--    Check that the rules for checking restrictions are followed.  Also check
--    that the No_Local_Allocators restriction allows instantiations of generic
--    packages, but that nested instantiations of such packages containing 
--    allocators are detected. (Defect Report 8652/0042).
--
-- APPLICABILITY CRITERIA:
--    This test is only applicable to implementations which support restriction
--    No_Local_Allocators.  This includes implementations which support the 
--    safety and security annex.
--
-- PASS/FAIL CRITERIA:
--      The test passes if:
--        A compile time error is generated indicating that the restriction
--          has been violated.
--        A linker error is generated because the restriction has been
--          violated.
--      The test fails if:
--        An executable image is linked.
--
-- CHANGE HISTORY:
--    12 FEB 2001   PHL   Initial version
--
--!
pragma Restrictions (No_Local_Allocators);
with Lxh4008A_0;
with Lxh4008A_1;
use Lxh4008A_0;
procedure Lxh4008A is

    function F (X : R := (D => True,  
			  C1 => '0',
			  C3 => new Integer'(0))) -- OK
	       return Boolean is separate;

    procedure P (X : Boolean := (D => True,  
				 C1 => '0',
				 C3 => new Integer'(0)) -- OK
				= Y) is separate;

    package Ip1 is new P1; -- OPTIONAL ERROR
    package Ip2 is new P2; -- OPTIONAL ERROR
    package Ip3 is new P3; -- OPTIONAL ERROR
    package Ip4 is new P4; -- OPTIONAL ERROR
    package Ip5 is new P5; -- OPTIONAL ERROR

    X1 : R; -- OPTIONAL ERROR
    X2 : R (False); -- OPTIONAL ERROR
    X3 : R (True); -- OPTIONAL ERROR

    B : Boolean;

    Y : Nt := (T with C2 => 0.0); -- OPTIONAL ERROR
    Z : Nt := (C1 => (D => False, C1 => 'o', C2 => 3.0), C2 => 4.0); -- OK

    task Nested;

    task body Nested is

	package Ip1 is new P1; -- OPTIONAL ERROR
	package Ip2 is new P2; -- OPTIONAL ERROR
	package Ip3 is new P3; -- OPTIONAL ERROR
	package Ip4 is new P4; -- OPTIONAL ERROR
	package Ip5 is new P5; -- OPTIONAL ERROR

	X1 : R; -- OPTIONAL ERROR
	X2 : R (False); -- OPTIONAL ERROR
	X3 : R (True); -- OPTIONAL ERROR

	B : Boolean;

	Y : Nt := (T with C2 => 0.0); -- OPTIONAL ERROR
	Z : Nt := (C1 => (D => False, C1 => 'o', C2 => 3.0), C2 => 4.0); -- OK
    begin
	B := F; -- OPTIONAL ERROR
	B := F ((D => False, C1 => 'B', C2 => -8.0)); -- OK
	P; -- OPTIONAL ERROR
	P (True); -- OK
    end Nested;

begin
    B := F; -- OPTIONAL ERROR
    B := F ((D => False, C1 => 'B', C2 => -8.0)); -- OK
    P; -- OPTIONAL ERROR
    P (True); -- OK
end Lxh4008A;

pragma Restrictions (No_Local_Allocators);
package Lxh4008A_0 is
    pragma Elaborate_Body;

    type Acc is access Integer;

    type R (D : Boolean := False) is
	record
	    C1 : Character := 's';
	    case D is
		when False =>
		    C2 : Float := 19.0;
		when True =>
		    C3 : Acc := new Integer'(7);
	    end case;
	end record;

    type T is tagged
	record
	    C1 : R (True);
	end record;

    type Nt is new T with
	record
	    C2 : Float;
	end record;

    Y : R; -- OK

    generic
    package P1 is
	X : R; -- OK
    end P1;

    generic
    package P2 is
    end P2;

    generic
    package P3 is
	X : Acc := new Integer'(5); -- OK
    end P3;

    generic
    package P4 is
    end P4;

    generic
	X : Acc := new Integer'(9); -- OK
    package P5 is
    end P5;

end Lxh4008A_0;

package body Lxh4008A_0 is

    package body P2 is
	X : R;
    end P2;

    package body P4 is
	Y : Acc := new Integer'(9);
    end P4;

end Lxh4008A_0;

pragma Restrictions (No_Local_Allocators);
with Lxh4008A_0;
use Lxh4008A_0;
package Lxh4008A_1 is

    package I1 is new P1; -- OK
    package I2 is new P2; -- OK
    package I3 is new P3; -- OK
    package I4 is new P4; -- OK
    package I5 is new P5; -- OK

end Lxh4008A_1;
